/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */ 

package ws.ladder.annotation;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import ws.ladder.model.ContentDetail;

/**
  * Use this annotation to include/exclude information as part of the BASE scope. 
  * If including information, then the detail option needs to be specified
  */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.TYPE)
public @interface Base {    
    ContentDetail detail() default ContentDetail.BASIC;  // This data will be included in all scopes. This option required if include is true.
    boolean include() default true;                      // Set to false if the type should not be part of the BASE scope.
}