/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */ 

package ws.ladder.annotation;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import ws.ladder.model.AggregateAction;

/**
 * This annotation can only be specified on a public method that
 * has a single argument
 */  
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface PrePropertyLogic {
	AggregateAction[] action() default {}; // The actions for which the annotation is valid. An empty action means it is valid for any.
	String[] tag() default {};             // The tag for which the annotation is valid. An empty tag means it is always invoked.
	String property();                     // The name of the property
	boolean capture() default false;       // If true, then the processing does not descend further
}