/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.model;


public abstract class AbstractTypeMapper implements TypeMapper {
	private MapperDirection direction;

	@Override
	public MapperDirection getDirection() {
		return direction;
	}
	
	@Override
	public void setDirection(MapperDirection direction) {
		this.direction = direction;
	}	
	
	@Override
	public EntityKey getEntityKey(Object identifier, Type type) {
		String referenceTypeName = type.getName();
		String derivedTypeName = type.getName();
		if(DerivedType.class.isAssignableFrom(type.getClass()))
			referenceTypeName = toReference(type.getInstanceClass()).getName();
		else
			derivedTypeName = toDerived(type.getInstanceClass()).getName();
		
		return new EntityKey(identifier, referenceTypeName, derivedTypeName);
	}
	
	@Override
	public boolean isDerived(Class<?> clazz) {
		return true;
	}
	
	@Override
	public boolean isReference(Class<?> clazz) {
		return true;
	}	
}
