/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import ws.ladder.util.ObjectCreator;

public class DataObjectList {

	private ExtendedDataObject dataObject;
	private Property property;

	public List<ExtendedDataObject> list(CallInfo callInfo) {
		Object list = (property != null) ? ((ExtendedProperty)property).getValue(dataObject) : dataObject.getInstance();
		if(list == null)
			return new ArrayList();

		Collection<Object> collection = null;
		if(Collection.class.isAssignableFrom(list.getClass()))
			collection = (Collection<Object>) list;
		else if(Map.class.isAssignableFrom(list.getClass()))
			collection = ((Map<Object, Object>) list).values();
		else
			//throw new IllegalArgumentException("Cannot create a DataObjectList for a non-list property: " + property.getName() + ", with type: " + property.getType().getName());
			return new ArrayList();

		Type type = (property != null) ? property.getType() : ((ExtendedProperty)dataObject.getContainmentProperty()).getElementType();		
		List<ExtendedDataObject> result = new ArrayList<ExtendedDataObject>();
		ObjectCreator objectCreator = dataObject.getObjectCreator();
		for(Object element: collection) {
			if(callInfo == null)
				result.add(objectCreator.createDataObject(element, type, (ExtendedDataObject) dataObject, null));
			else {
				CallInfo child = new CallInfo(null, null, callInfo, null);
				result.add((ExtendedDataObject) objectCreator.createSource(child, element, null));
			}
		}

		return result;
	}

	public DataObjectList(ExtendedDataObject dataObject, Property property) {
		this.dataObject = dataObject;
		this.property = property;
	}

	/**
	 * This constructor is for DataObject representing a collection
	 * @param dataObject
	 */
	public DataObjectList(ExtendedDataObject dataObject) {
		this.dataObject = dataObject;
	}	
}
