/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.model;

import java.util.Set;

public interface Node {
	
	/**
	 * A node is considered a dependent if it has incoming edges and one of them has the cascade flag set to true
	 * @return
	 */
	public boolean isDependent();
	
	/**
	 * Checks if self or any of its dependants are modified
	 * @return
	 */
	public boolean isAggregateModified();
	
	public void setAggregateName(String name);
	
	public String getAggregateName();
	
	public boolean isVisited();
	
	public void setVisited(boolean visited);	

	public Set<Edge> getOutgoing();
	
	public Set<Edge> getIncoming();	
	
	public Object getInstance();

	public void addIncomingEdge(Edge edge);

	public void removeIncomingEdge(Edge edge);

	public void addOutgoingEdge(Edge edge);

	public void removeOutgoingEdge(Edge edge);

	public void setAggregateSeal(String aggregateName);

	public void replaceInstance(Object newInstance);
	
	public boolean isCollectionNode();	

	public Edge addIncomingEdge(Node from, Property property);
	
	public Edge getOwner();

	boolean isPersistent();

	boolean isModified();

	void setModified(boolean modified);
}