/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.model.action;

import ws.ladder.model.CallInfo;
import ws.ladder.model.ExtendedDataObject;

public abstract class AbstractElementAction implements ElementAction {

	protected final PropertyKey key;
	protected final ExtendedDataObject collectionElement;	
	protected final ExtendedDataObject collectionOwner;
	protected final Executable triggeringAction;
	protected final Object position;
	protected final CallInfo callInfo;

	public AbstractElementAction(PropertyKey key, ExtendedDataObject collectionElement, ExtendedDataObject collectionOwner, Executable triggeringAction, Object position, CallInfo callInfo) {
		this.key = key;
		this.collectionElement = collectionElement;
		this.collectionOwner = collectionOwner;
		this.triggeringAction = triggeringAction;
		this.position = position;
		this.callInfo = callInfo;
	}

	@Override
	public void execute() {

	}	

	@Override
	public PropertyKey getKey() {
		return key;
	}

	@Override
	public ExtendedDataObject getCollectionElement() {
		return collectionElement;
	}
	
	@Override
	public ExtendedDataObject getCollectionOwner() {
		return this.collectionOwner;
	}

	@Override
	public Executable getTriggeringAction() {
		return this.triggeringAction;
	}

	@Override
	public Object getPosition() {
		return this.position;
	}

	@Override
	public CallInfo getCallInfo() {
		return this.callInfo;
	}			
}
