/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.model.action;

import ws.ladder.model.ExtendedDataObject;
import ws.ladder.model.ExtendedProperty;
import ws.ladder.model.Settings;
import ws.ladder.model.operation.AbstractOperation;


public final class SetterAction implements Executable {

	private final Object   value;
	private final PropertyKey key;
	private final Executable triggeringAction;

	public SetterAction(Object value, PropertyKey key, Executable triggeringAction) {
		this.value = value;
		this.key = key;
		this.triggeringAction = triggeringAction;
	}

	@Override
	public void execute() {
		setValue(value);
		AbstractOperation.setModified(key.getDataObject());	
	}
	
	public void setValue(Object value) {
		ExtendedProperty prop = ((ExtendedProperty)key.getProperty());
		
		ExtendedDataObject invokeOn = key.getDataObject();
		// First check if this property is part of this data object
		if(ExtendedDataObject.class.isAssignableFrom(invokeOn.getClass())) {
			if(((ExtendedDataObject)invokeOn).getType().getProperty(prop.getName()) == null && prop.getMapPath() != null && prop.getMapPath().contains(Settings.PATH_DELIMITER)) {
				// Get the embedded object
				invokeOn = (ExtendedDataObject) ((ExtendedDataObject)invokeOn).get( prop.getMapPath().substring(0, prop.getMapPath().lastIndexOf(Settings.PATH_DELIMITER)));
			}
		}
		
		prop.setValue(invokeOn, value);
	}		

	@Override
	public PropertyKey getKey() {
		return key;
	}

	@Override
	public Executable getTriggeringAction() {
		return triggeringAction;
	}	

	public Object getValue() {
		return value;
	}		
}
