/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.model.custom;


import ws.ladder.model.CallInfo;
import ws.ladder.util.ObjectCreator;


public class DefaultAssociationStrategy implements AssociationStrategy {
	/**
	 * The strategy used to associate with an existing object based on the source information
	 * @param callInfo
	 * @throws Exception 
	 */
	public Object execute(CallInfo callInfo, ObjectCreator oc) {
		return oc.getPersistenceOrchestrator().getPersistentObject(callInfo, oc.getTypeMapper());
	}

	/**
	 * Method that indicates if the source object should be persisted
	 * @param callInfo
	 */
	public boolean doProcess(CallInfo callInfo) {
		if(callInfo.getOperation().isExternalAssociationLink(callInfo))
			return false;
		
		return true;
	}
}
