/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.model.view;

import java.io.InputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

public class ContentViewFactory {
	private static final Logger logger = LogManager.getLogger(new Exception().getStackTrace()[0].getClassName());
	private static String CONTENT_VIEW_FILE = "ContentViews.xml";

	public void load() {
		load(CONTENT_VIEW_FILE);
	}
	
	public void load(String fileName) {
		ContentViews views = new ContentViews();

		try {
			InputStream stream = this.getClass().getClassLoader().getResourceAsStream(fileName);
			if(stream == null) {
				logger.warn("Unable to find the view configuration file: " + fileName);
				return;
			}
			
			JAXBContext jaxbContext = JAXBContext.newInstance(ContentViews.class);
			Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();

			views = (ContentViews) jaxbUnmarshaller.unmarshal(stream);
			views.sync();
			
		} catch (JAXBException e) {
			throw new RuntimeException("Unable to read " + fileName, e);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}