/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.model.view;

import java.util.Map;

import ws.ladder.model.Settings;
import ws.ladder.model.view.expression.AbstractFunctionExpression;
import ws.ladder.model.view.expression.ExpressionFactory;

public class Filter {

	protected String expression;

	protected AbstractFunctionExpression functionExpression;

	public AbstractFunctionExpression getFunctionExpression() {
		return functionExpression;
	}

	public void setFunctionExpression(AbstractFunctionExpression functionExpression) {
		this.functionExpression = functionExpression;
	}

	public Filter narrow() {
		Filter result = new Filter();
		result.setExpression(expression);
		result.init();

		return result;
	}

	public void init() {
		this.functionExpression = ExpressionFactory.getExpression(expression);
		this.functionExpression.init();
	}

	public boolean isOrderBy() {
		return functionExpression.isOrderBy();
	}

	public String getQueryString() {
		return functionExpression.getQueryString();
	}

	public String getNormalizedName() {
		return functionExpression.getNormalizedAttributeName();
	}

	public String getExpression() {
		return expression;
	}

	public void setExpression(String expression) {
		this.expression = expression;
	}

	public String getAttribute() {
		return functionExpression.getAttributeName();
	}

	public void normalize(Map<String, String> normalizedNames) {
		functionExpression.normalize(normalizedNames);
	}

	public Object getNormalizedValue(Object object) {
		return functionExpression.getNormalizedValue(object);
	}	

	public boolean updateFilters(Map<String, Object> originalFilters, Map<String, Object> filters, Map<String, Parameter> parameterMap) {
		boolean result = false;
		
		for(String parameterName: functionExpression.getParameters()) {

			Parameter param =  parameterMap.get(parameterName);
			if(!originalFilters.containsKey(Settings.encodeParam(parameterName)) ) { // parameter is not set
				// check if the parameter has a default value
				if(param != null)
					filters.put( Settings.encodeParam(parameterName), getNormalizedValue(param.defaultValue));
				else
					return result;
			} else {
				String key = Settings.encodeParam(parameterName);
				if(param != null)
					key = param.filterName;
				Object normalizedValue = getNormalizedValue(originalFilters.get(Settings.encodeParam(key)));
				filters.put( Settings.encodeParam(parameterName), normalizedValue);
			}
		}

		return true;
	}
}
