/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.service;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import ws.ladder.model.AggregateAction;
import ws.ladder.model.CallInfo;
import ws.ladder.model.ExtendedDataObject;
import ws.ladder.model.ExtendedProperty;
import ws.ladder.model.ExtendedType;
import ws.ladder.model.Property;
import ws.ladder.model.TypeMapper;

public abstract class AbstractPersistenceOrchestrator implements PersistenceOrchestrator {

	@Override
	public boolean supportsVersionTracking() {
		return true;
	}	
	
	@Override
	public boolean canProcessAggregate() {
		return false;
	}

	private Object getByUserKey(CallInfo callInfo, ExtendedType type) {
		ExtendedDataObject from = (ExtendedDataObject) callInfo.getInput();
		Property userKeyProperty = type.getUserKey();

		if(userKeyProperty != null) {
			Map<String, String> param = new HashMap<String, String>();
			if(from.get(userKeyProperty) == null)
				return null;
			
			param.put(userKeyProperty.getName(), from.get(userKeyProperty).toString() );

			return findByProperty(from.getReferenceType(), param);
		} else
			return null;
	}	
	
	@Override
	public Object getPersistentObject(CallInfo callInfo, TypeMapper typeMapper) {
		ExtendedDataObject from = (ExtendedDataObject) callInfo.getInput();
		Object persistentObject = null;

		if(!ExtendedType.class.isAssignableFrom(from.getType().getClass()))
			return null;

		ExtendedType type = (ExtendedType) from.getType();

		if(type.isEmbedded()) // We don't separately load embedded values from the database
			return null;

		if(!(callInfo.getSettings().getAction() == AggregateAction.CLONE) )
			persistentObject = getByUserKey(callInfo, type);

		if(persistentObject == null) {
			ExtendedProperty identifierProperty = (ExtendedProperty) type.getIdentifierProperty();

			Serializable id = (Serializable) identifierProperty.getValue(from);
			if(id != null) {
				Class<?> desiredClass = typeMapper.toReference(type.getInstanceClass());
				persistentObject = findById(desiredClass, id);
			} 
		}

		return persistentObject;
	}
}