/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.service;

import java.util.List;

import ws.ladder.model.Type;
import ws.ladder.model.TypeMapper;
import ws.ladder.model.view.QueryBuilder;
import ws.ladder.util.AccessType;

/**
 * 
 * @author Dilip Dalton
 *
 */
public interface DataAccessService {
	/**
	 * Build the Type and Property objects for the static API
	 */
	public void define();
	
	/**
	 * Return the type associated with a particular DataObject
	 * @param The java class whose Type object we need
	 * @return The Type object
	 */
	public Type getType(Class<?> clazz);
	
	/**
	 * Returns the Derived type for this class
	 * @param clazz
	 * @return
	 */
	public Type getDerivedType(Class<?> clazz);		
	
	/**
	 * Get the type first by looking for the class name and then fall-back to the entity name
	 * @param name
	 * @return
	 */
	public Type getType(String name);

	/**
	 * Return the derived type with the specific name
	 * @param name
	 * @return
	 */
	public Type getDerivedType(String name);
	
	/**
	 * Return all the types managed by this service
	 */
	public List<Type> getTypes();
	
	/**
	 * This method uses the persistence mechanism to do any post-processing activity
	 * @param newInstance
	 * @return
	 */
	public void postProcess(Object newInstance);
	
	/**
	 * The mapper used for creating the derived types
	 * @return
	 */
	public TypeMapper getTypeMapper();
	
	/**
	 * Gets the builder responsible for forming queries
	 * @return
	 */
	public QueryBuilder getQueryBuilder();
	
	/**
	 * Lists the names of the aggregates
	 * @return
	 */
	public List<String> getAggregateList();
	
	/**
	 * Returns the access type
	 */
	public AccessType getAccessType();
}