/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.service;

import java.lang.reflect.Method;

import javax.annotation.Resource;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.hibernate.cfg.Configuration;

import ws.ladder.model.TypeMapper;

/**
 * This class is part of the Data Access Service framework
 * 
 * @author Dilip Dalton
 * 
 */
public class Hibernate4DAS extends HibernateDAS {

	private static final Logger logger = LogManager.getLogger(new Exception()
	.getStackTrace()[0].getClassName());
	
	@Resource(name = "&sessionFactory")
	protected Object sessionFactoryBean;
	
	public Hibernate4DAS(TypeMapper typeMapper, DASFactory dasFactory) {
		super(typeMapper, dasFactory);
	}
	
	@Override
	public Configuration getConfiguration()  {
		if(configuration == null) {
			try {
				Method getConfigMethod = sessionFactoryBean.getClass().getMethod("getConfiguration", new Class[0]);
				Configuration config = (Configuration) getConfigMethod.invoke(sessionFactoryBean, new Object[0]);
				setConfiguration(config);
			} catch (Exception e) {
				logger.error("Unable to find the getConfiguration method in the session factory object");
			}
		}
		
		return configuration;
	}
}