/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.util;

import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import ws.ladder.model.Property;
import ws.ladder.model.SimpleType;
import ws.ladder.model.Type;
import ws.ladder.util.DFAtoRE.LiteralExpression;
import ws.ladder.util.DFAtoRE.State;

public class AggregatePropertyPaths {
	private static final Logger logger = LogManager.getLogger(new Exception().getStackTrace()[0].getClassName());	

	private static Map<Type, Set<String>> typePaths = new ConcurrentHashMap<Type, Set<String>>();   
	
	public static Set<String> enumerate(Type aggregateType) {
		Set<String> paths = typePaths.get(aggregateType);

		if(!typePaths.containsKey(aggregateType)) { 
			DFAtoRE dfaRE = new DFAtoRE(aggregateType);
			Map<DFAtoRE.State, DFAtoRE.Expression> expressions = dfaRE.getRegEx();

			paths  = new HashSet<String>();
			for(State state: expressions.keySet()) {
				Type type = state.getType();
				String prefix = expressions.get(state).toString();
				if(prefix.equals(LiteralExpression.EMPTY_STRING.toString()))
					prefix = "";
				for(Property childProperty: type.getProperties()) {
					if(childProperty != null && !childProperty.isMany() && SimpleType.class.isAssignableFrom(childProperty.getType().getClass())) {
						String path = prefix.concat(childProperty.getName());
						paths.add(path);
					}
				}
			}

			typePaths.put(aggregateType, paths);
		}

		return Collections.unmodifiableSet(paths);
	}
}