/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.util.btree;

import java.util.HashMap;
import java.util.Map;

public enum IndexTypeEnum {
    STRING ((byte)0),
    LONG ((byte)1);
    
    private static final Map<Byte, IndexTypeEnum> byteToType = new HashMap<Byte, IndexTypeEnum>();
    static {
        for (IndexTypeEnum type : IndexTypeEnum.values()) {
        	byteToType.put(type.getIndex(), type);
        }
    }

    public static IndexTypeEnum getType(int i) {
    	return byteToType.get(Byte.valueOf((byte)i));
    }     

    private final byte index;   

    IndexTypeEnum(byte index) {
        this.index = index;
    }

    public byte getIndex() { 
        return index; 
    }   
}