/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.util.btree;

public class MemoryStore extends AbstractStore {
	public static final int DEFAULT_BLOCK_SIZE = 8192;

	public MemoryStore(int entrySize, IndexTypeEnum type) {
		super(DEFAULT_BLOCK_SIZE, entrySize, type);
	}	

	public MemoryStore(int blockSize, int entrySize, IndexTypeEnum type) {
		super(blockSize, entrySize, type);
	}

	@Override
	public void save(Node x) {

	}

	@Override
	public void load(Node x) {
		if(x.getAddress() == 0) {
			x.setLeaf(true);
			x.setAddress(System.identityHashCode(x)); // Give it a unique address
		}
		cachedNodes.put(x.getAddress(), x);		
	}

	@Override
	public int getHeaderSize() {
		return 0;
	}

	@Override
	public IndexType getIndexType() {
		if(this.type == IndexTypeEnum.STRING)
			return new StringIndexType(entrySize);

		return null;
	}

	@Override
	public void setRoot(Node root) {
		rootAddress = root.getAddress();		
	}
}