/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations 
 * under the License.
 */

package ws.ladder.util.btree;

public interface Node {
	public static final int NODE_HEADER_LEN = Integer.SIZE/8 + Long.SIZE/8 + Byte.SIZE/8; // header has length of numKeys, offset and leaf datatypes	
	public static final int CHILD_PTR_LEN = Long.SIZE/8; // offset datatype length

	public byte[] getData();

	public long getAddress();

	public void setAddress(long value);
	
	public boolean isLeaf();
	
	public void setLeaf(boolean value);

	/**
	 * Event handler triggered after the data is loaded
	 */
	public void postLoad();

	/**
	 * Event handler triggered before the save is performed giving the chance for the client to initialize fields
	 */
	public void preSave();

	/**
	 * The number of keys currently in the node
	 * @return
	 */
	public int getNumKeys();
	
	/**
	 * Sets the number of keys in the node
	 * @param i
	 */
	public void setNumKeys(int i);	

	/**
	 * Returns the child node as the ith position. The position starts from 1.
	 * @param i
	 * @return
	 */
	public Node getChild(int i);

	/**
	 * Sets the child node at the ith position.
	 * @param i
	 * @param child
	 */
	public void setChild(int i, Node child);	
	
	/**
	 * Get the get at the ith position. The position starts from 1.
	 * @param keyIndex
	 * @return
	 */
	public Key getKey(int i);

	/**
	 * Set the key at the ith position
	 * @param keyIndex
	 * @param key
	 */
	public void setKey(int i, Key key);

	/**
	 * remove the ith key and i+1th child pointer from the node
	 * @param i
	 */	
	public void removeKey(int i);
}